"{ Package: 'stx:libjava/tools' }"

PPCompositeParser subclass:#JavaParserI
	instanceVariableNames:'unicodeEscape rawInputCharacter unicodeMarker hexDigit
		lineTerminator unicodeInputCharacter inputElements sub
		inputElement whiteSpace comment javaToken keyword literal
		separator operator identifier traditionalComment endOfLineComment
		commentTail charactersInLine commentTailStar notStar
		notStarNotSlash inputCharacter booleanLiteral nullLiteral
		keywords floatingPointLiteral integerLiteral characterLiteral
		stringLiteral input operators separators block'
	classVariableNames:''
	poolDictionaries:''
	category:'Languages-Java-Parser'
!


!JavaParserI class methodsFor:'instance creation'!

new
    "return an initialized instance"

    ^ self basicNew initialize.
! !

!JavaParserI class methodsFor:'accessing'!

ignoredNames
	"Answer a collection of instance-variables that should not be automatically initialized with productions, but that are used internal to the composite parser."

	| newArray |	
	newArray := Array new: ((self namesToIgnore size) + (super ignoredNames size)).
	newArray
		replaceFrom: 1
		to: self namesToIgnore size
		with: self namesToIgnore.
	newArray
		replaceFrom: (self namesToIgnore size + 1)
		to: newArray size
		with: super ignoredNames.	
	^newArray
!

namesToIgnore

	^#('keywords' 'operators' 'separators')
! !

!JavaParserI methodsFor:'accessing'!

builder

    ^nil

    "Created: / 10-03-2012 / 12:17:30 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

start
	"Default start production."

	^ input end
! !

!JavaParserI methodsFor:'grammar-blocks'!

block
    "Do no parse content of the block"

    ^JavaBlockParser new    

"/    ^ (self tokenFor: '{') , (self tokenFor: '}')

    "Created: / 11-03-2012 / 13:20:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!JavaParserI methodsFor:'grammar-comments'!

charactersInLine   

	^ inputCharacter plus
!

comment
	"traditional -> /*
	 endOfLine -> //"
	^ traditionalComment / endOfLineComment
!

commentTail

	^ 	('*' asParser , commentTailStar ) /
		(notStar , commentTail)
!

commentTailStar 

	^ ('/' asParser ) /
	  ('*' asParser , commentTailStar ) /
	  (notStarNotSlash , commentTail )
!

endOfLineComment 

        ^ 'cxcxcx' asParser

    "Modified: / 14-03-2012 / 23:13:39 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

notStar

	^ lineTerminator / ('*' asParser not , inputCharacter)
!

notStarNotSlash  

	^ lineTerminator / ((PPPredicateObjectParser anyOf: '*/') not , inputCharacter )
!

traditionalComment

        ^ 'aaaaa' asParser

    "Modified: / 14-03-2012 / 23:13:51 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!JavaParserI methodsFor:'grammar-identifiers'!

identifier

    ^ JavaTokenParser for: #Identifier


"/    ^  self asToken: (((keyword not) , (booleanLiteral not) , (nullLiteral not) , identifierChars ))

    "Modified: / 14-03-2012 / 23:10:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!JavaParserI methodsFor:'grammar-input'!

input

	^ (inputElements optional) , (sub optional)
!

inputElement

	^ whiteSpace / comment / javaToken
!

inputElements

	^ inputElement plus
!

javaToken
        "/self flag: 'TODO: check order'.

        ^ keyword / literal / separator / operator / identifier

    "Modified: / 23-08-2011 / 00:17:29 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

sub

	^ (Character value: 26) asParser 
! !

!JavaParserI methodsFor:'grammar-keywords'!

abstractKW
    ^ JavaTokenParser for:'abstract'
!

assertKW
    ^ JavaTokenParser for:'assert'
!

booleanKW
    ^ JavaTokenParser for:'boolean'
!

breakKW
    ^ JavaTokenParser for:'break'
!

byteKW
    ^ JavaTokenParser for:'byte'
!

caseKW
    ^ JavaTokenParser for:'case'
!

catchKW
    ^ JavaTokenParser for:'catch'
!

charKW
    ^ JavaTokenParser for:'char'
!

classKW
    ^ JavaTokenParser for:'class'
!

constKW
    ^ JavaTokenParser for:'const'
!

continueKW
    ^ JavaTokenParser for:'continue'
!

defaultKW
    ^ JavaTokenParser for:'default'
!

doKW
    ^ JavaTokenParser for:'do'
!

doubleKW
    ^ JavaTokenParser for:'double'
!

elseKW
    ^ JavaTokenParser for:'else'
!

enumKW
    ^ JavaTokenParser for:'enum'
!

extendsKW
    ^ JavaTokenParser for:'extends'
!

finalKW
    ^ JavaTokenParser for:'final'
!

finallyKW
    ^ JavaTokenParser for:'finally'
!

floatKW
    ^ JavaTokenParser for:'float'
!

forKW
    ^ JavaTokenParser for:'for'
!

gotoKW
    ^ JavaTokenParser for:'goto'
!

ifKW
    ^ JavaTokenParser for:'if'
!

implementsKW
    ^ JavaTokenParser for:'implements'
!

importKW
    ^ JavaTokenParser for:'import'
!

instanceofKW
    ^ JavaTokenParser for:'instanceof'
!

intKW
    ^ JavaTokenParser for:'int'
!

interfaceKW
    ^ JavaTokenParser for:'interface'
!

keyword

        | keywordParsers keywordParser |
        
        keywordParsers := keywords keys asSortedCollection "/SortedSafely 
                                                                collect: [:eachKey | keywords at: eachKey ].
        keywordParser := nil.
        keywordParsers do:[:each|
            keywordParser notNil ifTrue:[
                keywordParser := keywordParser / each
            ] ifFalse:[
                keywordParser := each
            ]
        ].

        ^ "self asToken: "keywordParser

    "Modified: / 09-03-2012 / 20:48:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

longKW
    ^ JavaTokenParser for:'long'
!

nativeKW
    ^ JavaTokenParser for:'native'
!

newKW
    ^ JavaTokenParser for:'new'
!

operator

        | operatorParsers operatorParser |
        
        operatorParsers := operators keys asSortedCollection "/SortedSafely 
                                                                collect: [:eachKey | operators at: eachKey ].
        operatorParser := nil.
        operatorParsers do:[:each|
            operatorParser notNil ifTrue:[
                operatorParser := operatorParser / each
            ] ifFalse:[
                operatorParser := each
            ]
        ].

        ^ "self asToken: "operatorParser

    "Modified: / 09-03-2012 / 20:50:35 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

packageKW
    ^ JavaTokenParser for:'package'
!

privateKW
    ^ JavaTokenParser for:'private'
!

protectedKW
    ^ JavaTokenParser for:'protected'
!

publicKW
    ^ JavaTokenParser for:'public'
!

returnKW
    ^ JavaTokenParser for:'return'
!

shortKW
    ^ JavaTokenParser for:'short'
!

staticKW
    ^ JavaTokenParser for:'static'
!

strictfpKW
    ^ JavaTokenParser for:'strictfp'
!

superKW
    ^ JavaTokenParser for:'super'
!

switchKW
    ^ JavaTokenParser for:'switch'
!

synchronizedKW
    ^ JavaTokenParser for:'synchronized'
!

thisKW
    ^ JavaTokenParser for:'this'
!

throwKW
    ^ JavaTokenParser for:'throw'
!

throwsKW
    ^ JavaTokenParser for:'throws'
!

transientKW
    ^ JavaTokenParser for:'transient'
!

tryKW
    ^ JavaTokenParser for:'try'
!

voidKW
    ^ JavaTokenParser for:'void'
!

volatileKW
    ^ JavaTokenParser for:'volatile'
!

whileKW
    ^ JavaTokenParser for:'while'
! !

!JavaParserI methodsFor:'grammar-lineTerminators'!

inputCharacter 

	^(lineTerminator not) , unicodeInputCharacter ==> #second
!

lineTerminator

	^ (Character lf asParser) / (Character cr asParser , (Character lf asParser ) optional )
! !

!JavaParserI methodsFor:'grammar-literals'!

literal
        "a literal must be a single token. Whitespaces are not allowed inside the literal"
        
        ^" self asToken: "(nullLiteral / booleanLiteral / floatingPointLiteral / integerLiteral / characterLiteral / stringLiteral)

    "Modified: / 09-03-2012 / 20:49:04 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!JavaParserI methodsFor:'grammar-literals-boolean'!

booleanLiteral 

    ^ (JavaTokenParser for: #boolean)

    "Modified: / 14-03-2012 / 23:36:55 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!JavaParserI methodsFor:'grammar-literals-character'!

characterLiteral 
    ^ (JavaTokenParser for: #Character)

    "Modified: / 15-03-2012 / 00:11:19 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (format): / 15-03-2012 / 08:35:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!JavaParserI methodsFor:'grammar-literals-floating'!

floatingPointLiteral
    ^ (JavaTokenParser for: #Float) /
            (JavaTokenParser for: #Double)

    "Modified: / 15-03-2012 / 00:03:12 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!JavaParserI methodsFor:'grammar-literals-integer'!

integerLiteral
    ^ (JavaTokenParser for: #Integer) /
            (JavaTokenParser for: #LongInteger)

    "Modified: / 15-03-2012 / 00:00:01 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!JavaParserI methodsFor:'grammar-literals-null'!

nullLiteral 

    ^ JavaTokenParser for: #null

    "Modified: / 14-03-2012 / 23:22:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!JavaParserI methodsFor:'grammar-literals-string'!

stringLiteral 

    ^ JavaTokenParser for: #String

    "Modified: / 14-03-2012 / 23:42:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!JavaParserI methodsFor:'grammar-separators'!

separator

        "should it be:
        
        | separatorParsers |
        separatorParsers := separators keysSortedSafely 
                                                                collect: [:eachKey | separators at: eachKey ].
        ^ (separatorParsers reduce: [ :a :b | a / b ]) token trimBlanks
        
        for consistency to keywords and operators ?"
        
        ^"self asToken: "(PPPredicateObjectParser anyOf: '(){}[];,.' )

    "Modified: / 09-03-2012 / 20:49:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!JavaParserI methodsFor:'grammar-unicode-escapes'!

hexDigit 

	^#hex asParser
!

rawInputCharacter

	^#any asParser
!

unicodeEscape

	^ $\ asParser , unicodeMarker , hexDigit , hexDigit , hexDigit , hexDigit
!

unicodeInputCharacter

	 ^ unicodeEscape / rawInputCharacter
!

unicodeMarker

	^$u asParser plus
! !

!JavaParserI methodsFor:'grammar-whiteSpace'!

whiteSpace

	^ (Character space asParser ) /
	  (Character tab asParser ) /
	  ((Character value: 12) asParser ) /
		lineTerminator 
! !

!JavaParserI methodsFor:'initialization'!

initialize

        super initialize.
        "/self initializeStartingAt: #start.
        
        self initializeKeywords.
        self initializeOperators.
        self initializeSeparators.

    "Modified: / 10-03-2012 / 12:13:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

initializeKeywords

        | values |
        keywords := Dictionary new.
        values := #('abstract' 'assert' 'boolean' 'break' 'byte' 'case'  'catch' 'char' 'class' 'const'
           'continue' 'default' 'do' 'double' 'else' 'enum' 'extends' 'final'  'finally' 'float'
           'for' 'if' 'goto' 'implements' 'import' 'instanceof' 'int' 'interface' 'long' 'native'
           'new' 'package' 'private' 'protected' 'public' 'return' 'short' 'static' 'strictfp' 'super'
           'switch' 'synchronized' 'this' 'throw' 'throws' 'transient' 'try' 'void' 'volatile' 'while').
        
        values do: [:eachKeyword |
                keywords at: eachKeyword 
                        put: (PPUnresolvedParser named: ('keyword', eachKeyword first asUppercase asString , eachKeyword allButFirst))          
                ].
        
        keywords keysAndValuesDo:  [:key :value |
                (keywords at: key) def: key asParser trim
        ]

    "Modified: / 10-03-2012 / 13:16:05 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

initializeOperators

        | values |
        operators := Dictionary new.
        values := #(    '>>>=' '>>>' '>>=' '>>' '>=' '>'        '<<=' '<<' '<=' '<'     '++' '+=' '+'   '--' '-=' '-'   '&&' '&=' '&'
                                        '||' '|=' '|'   '*=' '*'        '%=' '%'        '/=' '/'        '^=' '^'        '!!=' '!!'        '==' '='        '~'     '?'     ':'     '@' ).
        
        values do: [:eachOperator |
                operators at: eachOperator 
                        put: (PPUnresolvedParser named: ('operator', eachOperator asString))            
                ].
        
        operators  keysAndValuesDo:  [:key :value |
                (operators at: key) def: (key asParser trim)]

    "Modified: / 10-03-2012 / 13:16:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

initializeSeparators

        | values |
        separators := Dictionary new.
        values := #( '(' ')' '{' '}' '[' ']' ';' ',' '.' ).
        
        values do: [:eachSeparator |
                separators at: eachSeparator 
                        put: (PPUnresolvedParser named: ('separator', eachSeparator asString))          
                ].
        
        separators  keysAndValuesDo:  [:key :value |
                (separators at: key) def: (key asParser trim)]

    "Modified: / 10-03-2012 / 13:16:35 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

initializeStartingAt: aSymbol
        | allVariableNames ignoredVariableNames productionIndexesAndNames debugger |
        self initialize.        

        "/debugger := PPDebugger new.
        "find all the productions that need to be initialized"
        allVariableNames := self class allInstVarNames
                collect: [ :each | each asSymbol ].
        ignoredVariableNames := self class ignoredNames
                collect: [ :each | each asSymbol ].
        productionIndexesAndNames := ((1 to: self class instSize)
                collect: [ :index | index -> (allVariableNames at: index) ])
                reject: [ :assoc | ignoredVariableNames includes: assoc value ].

        "initialize productions with an undefined parser to be replaced later"
        parser := PPUnresolvedParser named: aSymbol.
        productionIndexesAndNames do: [ :assoc |
                self instVarAt: assoc key put: (PPUnresolvedParser named: assoc value) ].
        parser def: (self perform: aSymbol).

        "resolve unresolved parsers with their actual implementation"
        productionIndexesAndNames do: [ :assoc |
                (self respondsTo: assoc value)
                        ifFalse: [ self error: 'Unable to initialize ' , assoc value printString ]
                        ifTrue: [ (self instVarAt: assoc key) def: 
"/                                (PPDebuggingParser parser:
                                    ((self perform: assoc value)  name: assoc value; yourself)
"/                                    debugger: debugger)
                                ] ]

    "Created: / 12-03-2012 / 16:51:01 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!JavaParserI methodsFor:'parsing'!

parse: anObject
    "Parse anObject with the receiving parser and answer the parse-result or an instance of PPFailure."

    | scanner |

    (anObject isKindOf: JavaScanner) ifFalse:[
        scanner := JavaScanner for: anObject readStream.
    ] ifTrue:[
        scanner := anObject
    ].

    ^super parse: scanner

    "Created: / 15-03-2012 / 11:10:53 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!JavaParserI methodsFor:'querying'!

productionAt: aSymbol ifAbsent: aBlock
    "Answer the production named aSymbol, if there is no such production answer the result of evaluating aBlock."

    parser isNil ifTrue:[
        self initializeStartingAt: aSymbol.
    ].
    ^super productionAt: aSymbol ifAbsent: aBlock

    "Created: / 10-03-2012 / 12:15:29 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!JavaParserI methodsFor:'utility'!

asToken:aParser
    ^aParser

    "Created: / 11-03-2012 / 08:43:51 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

emptySquaredParenthesis

        ^ (self tokenFor:$[) , (self tokenFor:$])

    "Created: / 11-03-2012 / 00:02:09 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

tokenFor: aString

    ^aString size == 1 ifTrue:[
        JavaTokenParser for: aString first
    ] ifFalse:[
        JavaTokenParser for: aString
    ]


"/    | p |
"/
"/    p := keywords at: aString ifAbsent:[nil].
"/    p notNil ifTrue:[ ^ self asToken: p ].
"/
"/    p := operators at: aString ifAbsent:[nil].
"/    p notNil ifTrue:[ ^ self asToken: p ].
"/
"/    p := separators at: aString ifAbsent:[nil].
"/    p notNil ifTrue:[ ^ self asToken: p ].
"/
"/    self error: 'Should not be reached'

    "Modified: / 15-03-2012 / 08:53:08 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!JavaParserI class methodsFor:'documentation'!

version_SVN
    ^ '$Id$'
! !
