package stx.libjava.tools;

import java.util.ArrayList;

import org.eclipse.jdt.core.dom.*;

public class SyntaxHighlighter extends ASTVisitor {
	protected SyntaxHighlighterHandler handler;
	
	protected ArrayList l;
	

	public SyntaxHighlighter(SyntaxHighlighterHandler handler) {
		this.handler = handler;
	}
	
	public boolean visit(PackageDeclaration node) {
		handler.markKeyword(node.getStartPosition(), /*package*/7);
		return true;
	}
	
	public boolean visit(ImportDeclaration node) {
		handler.markKeyword(node.getStartPosition(), /*import*/6);
		return true;
	}
	
	public boolean visit(QualifiedName node) {
		handler.markClassName(node.getStartPosition(), node.getLength());
		return false;
	}
	
	public boolean visit(SimpleName node) {
		handler.markClassName(node.getStartPosition(), node.getLength());
		return false;
	}
	
	public boolean visit(NumberLiteral node) {
		handler.markConstant(node.getStartPosition(), node.getLength());
		return false;
	}
	
	
	
	public boolean visit(CompilationUnit node) {
		for ( Object comment : node.getCommentList()) {
			((ASTNode)comment).accept(this);
		}
		return true;
	}
	
	public boolean visit(StringLiteral node) {
		handler.markString(node.getStartPosition(), node.getLength());		
		return false;
	}
	
	public boolean visit(BlockComment node) {
		handler.markComment(node.getStartPosition(), node.getLength());
		return false;
	}
	
	public boolean visit(LineComment node) {
		handler.markComment(node.getStartPosition(), node.getLength());
		return false;
	}

	public boolean visit(Javadoc node) {
		handler.markComment(node.getStartPosition(), node.getLength());
		return false;
	}


	public void formatExpression(String source) {
		formatExpression(source.toCharArray());		
	}

	public void formatExpression(char[] source) {
		format(source, ASTParser.K_EXPRESSION);
	}
	
	public void format(char[] source, int kind) {		
		ASTParser parser = ASTParser.newParser(AST.JLS3);		
		parser.setSource(source);
		parser.setKind(kind);
		format(parser.createAST(null));
		
	}
	
	public void format(ASTNode node) {
		node.accept(this);
	}
	
	

}
